/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ScanClassVisitor.java 2234 2008-01-17 16:43:23Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.deployment.annotations.analyzer;

import java.util.ArrayList;
import java.util.List;

import org.ow2.easybeans.asm.ClassVisitor;
import org.ow2.easybeans.asm.FieldVisitor;
import org.ow2.easybeans.asm.MethodVisitor;
import org.ow2.util.deployment.annotations.analyzer.configurator.ArchiveConfigurator;
import org.ow2.util.deployment.annotations.analyzer.configurator.ClassConfigurator;
import org.ow2.util.deployment.annotations.analyzer.configurator.FieldConfigurator;
import org.ow2.util.deployment.annotations.analyzer.configurator.MethodConfigurator;
import org.ow2.util.deployment.metadata.structures.JClass;
import org.ow2.util.deployment.metadata.structures.JField;
import org.ow2.util.deployment.metadata.structures.JMethod;

/**
 * ScanClassVisitor.
 * @author Gael Lalire
 */
public final class ScanClassVisitor extends ScanCommonVisitor<ClassConfigurator> implements ClassVisitor {

    /**
     * list of archives.
     */
    private List<ArchiveConfigurator> archiveConfigurators;

    /**
     * Constructor.
     * @param archiveConfigurators list of archives
     */
    public ScanClassVisitor(final List<ArchiveConfigurator> archiveConfigurators) {
        this.archiveConfigurators = archiveConfigurators;
    }

    /**
     * visit the class.
     * @param version the version
     * @param access access (public, ...)
     * @param name class name
     * @param signature signature
     * @param superName super class name
     * @param interfaces implemented interfaces name
     */
    @Override
    public void visit(final int version,
                      final int access,
                      final String name,
                      final String signature,
                      final String superName,
                      final String[] interfaces) {
        List<ClassConfigurator> classConfigurators = new ArrayList<ClassConfigurator>();
        JClass jClass = new JClass(version, access, name, signature, superName, interfaces);
        for(ArchiveConfigurator archiveConfigurator : archiveConfigurators) {
            classConfigurators.add(archiveConfigurator.createClassConfigurator(jClass));
        }
        setCommonConfigurators(classConfigurators);
    }

    /**
     * @param access access of method (public, ...)
     * @param name name of method
     * @param desc desc
     * @param signature signature of method
     * @param exceptions throwed exceptions
     * @return a visitor
     */
    @Override
    public MethodVisitor visitMethod(final int access, final String name, final String desc, final String signature,
            final String[] exceptions) {
        JMethod jMethod = new JMethod(access, name, desc, signature, exceptions);
        List<MethodConfigurator> methodConfigurators = new ArrayList<MethodConfigurator>();
        for(ClassConfigurator classConfigurator : getCommonConfigurators()) {
            methodConfigurators.add(classConfigurator.createMethodConfigurator(jMethod));
        }
        return new ScanMethodVisitor(methodConfigurators);
    }

    /**
     * @param access access of field (public, ...)
     * @param name name of field
     * @param desc desc
     * @param signature signature of field
     * @param value value of field
     * @return a visitor
     */
    @Override
    public FieldVisitor visitField(final int access, final String name, final String desc, final String signature,
            final Object value) {
        JField jField = new JField(access, name, desc, signature, value);
        List<FieldConfigurator> fieldConfigurators = new ArrayList<FieldConfigurator>();
        for(ClassConfigurator classConfigurator : getCommonConfigurators()) {
            fieldConfigurators.add(classConfigurator.createFieldConfigurator(jField));
        }
        return new ScanFieldVisitor(fieldConfigurators);
    }


    /**
     * call configurationComplete on each configurator.
     */
    @Override
    public void visitEnd() {
        for(ClassConfigurator classConfigurator : getCommonConfigurators()) {
            classConfigurator.configurationComplete();
        }
    }
}
